﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ShizukuLib;
using System.IO.Ports;
using static ShizukuLib.ShizukuProtocolMeterData;
using static ShizukuLib.ShizukuProtocolFastCharge;
using static ShizukuLib.ShizukuProtocolLoadCon;
using static ShizukuLib.ShizukuProtocol;

namespace Shizuku_Lib_Example
{
    /*
      This is an example to communicate with Shiuzku USB Multimeter with Shizuku Lib.
    */

    public partial class TestForm : Form
    {
        ShizukuProtocol protocolStack;
        SerialPort targetPort;
        ShizukuProtocolMeterData pMeterData;
        ShizukuProtocolFastCharge pFastCharge;
        ShizukuProtocolLoadCon pLoadCon;
        USBMeterData_t USBMeterData_RealTime = new USBMeterData_t();

        private void MeterDataReportCallback(byte[] packet)
        {
            /*When a meter report packet is received, this method is called, it converts the packet into USBMeterData_RealTime*/
            byte[] payload = packet.Skip(9).Take(32).ToArray();
            ByteStructConverter byteStructConverter = new ByteStructConverter();
            USBMeterData_RealTime = (USBMeterData_t)byteStructConverter.BytesToStruct(payload, USBMeterData_RealTime.GetType());
        }

        ShizukuProtocol.CallerDescriptor.Callback_t mdr_cb;

        public TestForm()
        {
            InitializeComponent();
        } 

        private async void ConnectButton_Click(object sender, EventArgs e)
        {
            /* The following gives you a basic workflow of using the library.
             * After this, you should be able to fit the library into your own project.
            */

            /*First, you declare a SerialPort with the port No of the multimeter.
              Replace the port No to make the demo work.*/
            targetPort = new SerialPort("COM26");

            /*You pass the serial port object to construct a ShizukuProtocol object, which 
             handles low layer communications with the multimeter*/
            protocolStack = new ShizukuProtocol(targetPort);
            /*It's recommended to send a reset command before any operation*/
            await protocolStack.Reset();

            /*Then you can read the meter data*/
            pMeterData = new ShizukuProtocolMeterData(protocolStack);
            /*Specify a callback when a meter data report packet is received*/
            mdr_cb = new ShizukuProtocol.CallerDescriptor.Callback_t(MeterDataReportCallback);
            /*We request the meter to report meter data every 1000 miliseconds*/
            await pMeterData.MeterDataReport_Setup(1000, mdr_cb, false);

            /*This is how you trigger fastcharge.*/
            pFastCharge = new ShizukuProtocolFastCharge(protocolStack);
            await pFastCharge.FastChargeCommand(new UInt32[1] { (uint)FastChargeCommandType.CREATE_INTERFACE }, true);
            /*Send a command ENTER_MODE with a protocol paramater, we use QC2.0 as an example*/
            await pFastCharge.FastChargeCommand(
                new UInt32[2] { (uint)FastChargeCommandType.ENTER_MODE,
                (uint)FastChargeMode.QC_2P0}, false);
            /*Then you request to trigger a certain voltage, in this case we want 9V@3A*/
            await pFastCharge.FastChargeCommand(
                          new UInt32[3] { (uint)FastChargeCommandType.CHANGE_VALUE
                     ,9000
                     ,3000},false);
            /*You can plug-in a QC2.0 supported charger to see the result*/

            /*There is little documentation, contact +8618628228557 for further support*/
            /*If there is a demand, we will make a full documentation of more APIs.*/
            LoadControlButton.Enabled = true;
        }

        /*This is where you play with SM-LD-00 (Load module)*/
        bool loadStatus = false;
        bool loadInited = false;
        LoadConReport_t LoadConReport_Now;

        /*Callback when a load controller report message is received*/
        private void LoadConReportHandler(byte[] packet)
        {
            /*This convert the data packet into LoadConReport_Now, where you can access various status of the load module(temperature etc.)*/
            byte[] payload = packet.Skip(9).Take(12).ToArray();
            LoadConReport_Now = (LoadConReport_t)(new ByteStructConverter()).BytesToStruct(payload, typeof(LoadConReport_t));
        }
        private async void LoadControlButton_Click(object sender, EventArgs e)
        {
            if (loadInited == false)
            {
                pLoadCon = new ShizukuProtocolLoadCon(protocolStack);
                /*Only setup load controller when there is a module installed*/
                ShizukuProtocol.LoadConInfoReport info = await pLoadCon.GetLoadConInfo();
                if ((LoadConStatus)info.LoadConStatus != LoadConStatus.Ready)
                {
                    MessageBox.Show("Load con is not ready, check if any load module is installed. Error code: " + (LoadConStatus)info.LoadConStatus);
                    return;
                }
                byte[] r = await pLoadCon.LoadConCommand(LoadConCommandType.LOADCON_CMD_OPEN, new UInt32[0]);
                if (!protocolStack.IsRequestReplyType(GetPacketHeader(r), RequestReply.DONE))
                {
                    MessageBox.Show("Error while opening LoadCon: " + GetPacketRequestReplyType(GetPacketHeader(r)).ToString());
                    return;
                }

                await pLoadCon.LoadConReporter_Setup(new CallerDescriptor.Callback_t(LoadConReportHandler), true);

                /*We set a 1A constant current load (protection voltage 4V) for demo purpose*/
                await pLoadCon.LoadConCommand(LoadConCommandType.LOADCON_CMD_SETUP, new UInt32[3] { (int)LoadMode.ConstantCurrent,
                            ForceFloatAsUInt32(1.0f),
                            ForceFloatAsUInt32(4.0f)});
                loadInited = true;
            }
            if (loadInited)
            { 
                /*We toggle the ON/OFF status of the load module just for demo purpose*/
                await pLoadCon.LoadConCommand(loadStatus ? LoadConCommandType.LOADCON_CMD_TURNOFF : LoadConCommandType.LOADCON_CMD_TURNON, new UInt32[0]);
                loadStatus = !loadStatus;
            }
        }
    }
}

